@@ -793,9 +793,10 @@ on_ok_button_clicked (GtkWidget *widget,
 
 	button = glade_xml_get_widget (xml, "save_rbutton");
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
-		GtkWidget *entry;
-		GtkWidget *fileentry;
-		const gchar *tmp;
+		GtkWidget  *entry;
+		GtkWidget  *fileentry;
+		const char *tmp;
+
 		entry = glade_xml_get_widget (xml, "save_entry");
 		fileentry = glade_xml_get_widget (xml, "save_fileentry");
  		tmp = gtk_entry_get_text (GTK_ENTRY (entry));
@@ -807,6 +808,7 @@ on_ok_button_clicked (GtkWidget *widget,
 		}
 		g_free (file);
 		setup_busy (FALSE);
+
 		return;
 	}
 
@@ -1216,7 +1218,6 @@ main (int argc, char *argv[])
 	gint width, height; 
 	guint delay = 0;
 	gboolean normal_web_dir = TRUE;
-	gchar *utf8_name;
 	
 	struct poptOption opts[] = {
 		{"window", '\0', POPT_ARG_NONE, &window, 0, N_("Grab a window instead of the entire screen"), NULL},
@@ -1349,10 +1350,8 @@ main (int argc, char *argv[])
 	g_object_unref (gconf_client);
 	
 	file = add_file_to_path (home_dir);
-	utf8_name = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-	gtk_entry_set_text (GTK_ENTRY (save_entry), utf8_name);
+	gtk_entry_set_text (GTK_ENTRY (save_entry), file);
 	g_free (file);
-	g_free (utf8_name);
 
 	if (!stat (web_dir, &s) && S_ISDIR (s.st_mode)) {
 		GtkWidget *cbutton;
